<?php

/**
 * @version     $Id: attendance.php 9 2014-09-29 10:51:55Z david.dorchies $
 * @package     Joomla16.Tutorials
 * @subpackage  Components
 * @copyright   Copyright (C) 2005 - 2010 Open Source Matters, Inc. All rights reserved.
 * @author      Christophe Demko
 * @link        http://joomlacode.org/gf/project/umrmanagement_1_6/
 * @license     License GNU General Public License version 2 or later
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

// import Joomla modelform library
jimport('joomla.application.component.modeladmin');

/**
 * UmrManagement Model
 */
class UmrManagementModelAttendance extends JModelAdmin
{

    /**
         * Returns a reference to the a Table object, always creating it.
         *
         * @param       type    The table type to instantiate
         * @param       string  A prefix for the table class name. Optional.
         * @param       array   Configuration array for model. Optional.
         * @return      JTable  A database object
         * @since       2.5
         */
    public function getTable($type = 'Attendance', $prefix = 'UmrManagementTable', $config = array())
    {
        return JTable::getInstance($type, $prefix, $config);
    }

    /**
     * Method to get the record form.
     *
     * @param   array   $data       Data for the form.
     * @param   boolean $loadData   True if the form is to load its own data (default case), false if not.
     * @return  mixed   A JForm object on success, false on failure
     * @since   1.6
     */
    public function getForm($data = array(), $loadData = true)
    {
        // Get the form.
        $form = $this->loadForm('com_umrmanagement.attendance', 'attendance', array('control' => 'jform', 'load_data' => false));
        if (empty($form))
        {
            return false;
        }
        // Chargement du fieldset extra_infos (Les données ont déjà été chargées par loadFormData lancé par loadForm
        $form->load($this->getXMLExtraInfo(),true);

        // Chargement des données
        $data = $this->loadFormData();

        // Allow for additional modification of the form, and events to be triggered.
        // We pass the data because plugins may require it.
        $this->preprocessForm($form, $data);

        // Load the data into the form after the plugins have operated.
        $form->bind($data);

        return $form;
    }


    /**
     * Création dynamique du fichier XML de formulaire pour les informations complémentaires
     * @author David Dorchies
     * @date 23/08/2014
     */
    private function getXMLExtraInfo() {
        $Xml = new SimpleXMLElement('<form></form>');
        $XmlFS = $Xml->addChild('fieldset');
        $XmlFS->addAttribute('name','extra_infos');

        // Liste des champs pour la catégorie en cours
        $item = $this->getItem();
        $categorie = $item->get('categorie');
        if(is_numeric($categorie)) {
            require_once(JPATH_COMPONENT_SITE.DS.'models'.DS.'umrtools.php');
            $aChamps = UmrManagementModelUmrTools::getChampsExtraInfo($categorie);

            $aExtraInfos = array();
            $db = JFactory::getDbo();
            foreach($aChamps as $sChamp) {
                // Récupération des paramètres du champ
                $aExtraParam = UmrManagementModelUmrTools::getChampsExtraParam($sChamp);

                // Ajout des infos dans le XML
                $XmlField = $XmlFS->AddChild('field');
                $XmlField->AddAttribute('name',$sChamp);
                $XmlField->AddAttribute('type',$aExtraParam['type']);
                $XmlField->AddAttribute('label',$aExtraParam['libelle']);
                $XmlField->AddAttribute('class','');
                switch($aExtraParam['type']) {
                    case 'text':
                        $XmlField->AddAttribute('size','100');
                        break;
                    case 'umrlibelle':
                        $XmlField->AddAttribute('libelle_long','2');
                        break;
                }
            }
        } else {
            // TODO ? gérer l'erreur : catégorie absente
        }
        return $Xml->asXML();
    }

    //~ /**
     //~ * Get the message
     //~ * @return object The message to be displayed to the user
     //~ */
    //~ function &getItem()
    //~ {
        //~ if (!isset($this->_item))
        //~ {
            //~ $cache = JFactory::getCache('com_umrmanagement', '');
            //~ $id = $this->getState('attendance.idmvt');
            //~ $this->_item =  $cache->get($id);
            //~ if ($this->_item === false) {
//~
            //~ }
        //~ }
        //~ return $this->_item;
    //~ }


    /**
     * Method to get the data that should be injected in the form.
     * @author David Dorchies
     * @date 23/08/2014
     * @return  mixed   The data for the form.
     */
    protected function loadFormData()
    {
        // Check the session for previously entered form data.
        $mainframe = JFactory::getApplication();
        $data = $mainframe->getUserState('com_umrmanagement.edit.attendance.data', array());
        if (empty($data))
        {
            $data = $this->getItem();
        }
        if(is_object($data)) {
            // Transmission du champ id au formulaire pour les nouveaux enregistrements
            if(!is_numeric($data->get('id'))) {
                $data->set('id',$mainframe->getUserState('com_umrmanagement.attendances.id_membre',-1));
            }

            // Ajout des champs des informations complémentaires (fieldset extra_infos)
            require_once(JPATH_COMPONENT_SITE.DS.'models'.DS.'umrtools.php');
            $aExtraItem = UmrManagementModelUmrTools::getChampsExtraItem($data->get('idmvt'));
            foreach($aExtraItem as $nom=>$valeur) {
                $data->set($nom,$valeur);
            }
        }
        return $data;
    }


    /**
     * Method to save the form data.
     * @param   array  $data  The form data.
     * @return  boolean  True on success, False on error.
     * @since   11.1
     */
    public function save($data)
    {
        // Save attendance object
        if(!parent::save($data)) {
            return false;
        }
        if(!$data['idmvt']) {
            // Nouvel enregistrement, la catégorie n'a pas pu
            return true;
        }
        // Save extra infos
        // La function JModelForm::validate puis JForm::filter se base sur attendance.xml pour expurger les champs superflus
        // Il faut donc reprendre les données directement dans le post pour récupérer les champs complémentaires
        $data  = JRequest::getVar('jform', array(), 'post', 'array');
        $categorie = $data['categorie'];
        if(!is_numeric($categorie)) {
            // TODO SetError
            return false;
        }
        require_once(JPATH_COMPONENT_SITE.DS.'models'.DS.'umrtools.php');
        $aChamps = UmrManagementModelUmrTools::getChampsExtraInfo($categorie);
        $db = JFactory::getDbo();
        foreach($aChamps as $sChamp) {
            // Does the record exists in umrmanagement_enregistrements ?
            $query = $db->getQuery(true);
            $query->select('count(*)');
            $query->from($db->quotename('#__umrmanagement_enregistrements'));
            $query->where($db->quotename('idmvt').' = '.$data['idmvt'].
                        ' AND '.$db->quotename('nom').' = '.$db->quote($sChamp));
            $query->group($db->quotename(array('idmvt','nom')));
            $db->setQuery($query);
            $count = $db->loadResult();
            if($count !== null && $count > 0) {
                // Updating existing record
                $query = $db->getQuery(true);
                $fields = $db->quoteName('valeur') . ' = ' . $db->quote($data[$sChamp]);
                $conditions = array(
                    $db->quoteName('idmvt').' = '.$data['idmvt'],
                    $db->quoteName('nom').' = '.$db->quote($sChamp)
                );
                $query->update($db->quoteName('#__umrmanagement_enregistrements'))->set($fields)->where($conditions);
                $db->setQuery($query);
                $result = $db->query();
            } else {
                // Inserting new record
                $object = new stdClass();
                $object->idmvt = $data['idmvt'];
                $object->nom = $sChamp;
                $object->valeur = $data[$sChamp];
                $db->insertObject('#__umrmanagement_enregistrements', $object);
            }
        }
        return true;
    }
}
